import { Injectable } from '@angular/core';
import { LaravelEchoService } from '../../../core/laravel-echo.service';
import { AuthService } from '../../../core/auth/auth.service';
import { AuthHttp } from 'angular2-jwt';
import { env } from '../../../../../../../src/.env';
import { Response } from '@angular/http';
import * as moment from 'moment';
import {QuestionsService} from "../../../../../../../src/app/modules/questions/shared/questions.service";
import {Subject} from "rxjs/Subject";

@Injectable()
export class SidebarNotificationService {

    public notifications = [];
    public notificationTypes = [
        {
            type: 'App\\Notifications\\MarkAsReadNotification',
            action: function(obj, notification) {
                obj.addNotification(notification);
                obj.add(notification.ticket.id);
                this.fetchQuestions();
            }
        },
        {
            type: 'App\\Notifications\\NewTicketCreated',
            action: (ctx, notification) => {
                const sound = new Audio('/assets/coins.mp3');

                this.addAlert(notification.ticket.id);
                this.fetchNotifications();
                sound.play();
            }
        }
    ];

    protected isOpened: boolean = false;
    protected isShown: boolean = true;
    protected ticketNotificationsUrl = env('apiUrl') + '/flow/tickets/notifications';
    protected body = document.getElementsByTagName('body')[0];

    public alerts: any = [];

    private notificationObserver = new Subject<boolean>();

    private addAlert(id: number = 200): void {
        this.alerts.push({
            type: 'success',
            msg: `Добавлен новый вопрос № ${id}`,
            timeout: 30000,
            notificationId: id,
        });
    }

    constructor(protected echo: LaravelEchoService, protected auth: AuthService, protected http: AuthHttp ) {

        this.echo.echo.subscribe((Echo) => {
            const sound = new Audio('/assets/coins.mp3');

            if (Echo) {
                Echo.private('App.User.' + this.auth.getData().id)
                    .notification((notification) => {
                        this.notificationTypes.forEach((value)=>{
                            if (value.type === notification.type) {
                                value.action(this, notification);
                            }
                        });
                        this.sortNotifications();
                    });
            }
        });

        this.http.get(this.ticketNotificationsUrl).map((res: Response) => {
            let body = res.json();
            return body || { };
        }).subscribe((data) => {
            this.notifications.push(...data.data);
            this.sortNotifications();
        });
    }

    public showQuestion(id: number){
        // this.questionsService.resetParams();
        // this.questionsService.setParams({id: id});
        // this.questionsService.fetchQuestions();
    }

    public addNotification(notification) {
        this.notifications.push(notification);
    }

    public addNotificationType(notificationType){
        this.notificationTypes.push(notificationType);
    }

    public removeNotification(notificationId) {
        this.notifications = this.notifications.filter((notify) => notify.id !== notificationId);
    }

    public open() {
        if (this.notifications.length === 0) {
            return false;
        }
        this.isOpened = true;
        this.setClass();
    }

    public close() {
        this.isOpened = false;
        this.setClass();
    }

    public toggle() {
        this.isOpened = !this.isOpened;
        this.setClass();
    }

    public shown(): boolean {
        return this.isOpened && this.notifications.length > 0;
    }

    protected sortNotifications() {
        let format = 'YYYY-MM-DD HH:mm:ss';
        return this.notifications.sort((a, b) => moment(b.created_at, format).diff(moment(a.created_at, format)));
    }

    public deleteNotification(id: number){
        return this.http.delete(env('apiUrl') + '/flow/notification/' + id);
    }

    public deleteNotifications(){
        return this.http.delete( env('apiUrl') + '/flow/notifications' );
    }

    public onFetchNotifications() {
        return this.notificationObserver;
    }

    public fetchNotifications() {
        this.notificationObserver.next(true);
    }

    protected setClass() {
        if (this.isOpened) {
            document.body.classList.add('page-quick-sidebar-open');
        }
        if (!this.isOpened) {
            document.body.classList.remove('page-quick-sidebar-open');
        }
    }

}
