import { Injectable, OnInit } from '@angular/core';
import { tokenNotExpired, JwtHelper } from 'angular2-jwt';
import { Router } from '@angular/router';
import { Subject } from 'rxjs/Subject';

@Injectable()
export class AuthService {

    public isAuth = new Subject<boolean>();

    protected jwtHelper: JwtHelper = new JwtHelper();

    protected tokenName: string = 'token';
    protected data: any = null;

    private lastCheck = false;

    constructor(protected router: Router) {
    }

    public setAuth(isAuth: boolean = false) {
        this.isAuth.next(isAuth);
        return isAuth;
    }

    public getData() {
        if (!this.data && this.getToken()) {
            let payload = this.jwtHelper.decodeToken(this.getToken());
            this.data = payload.user;
        }
        return this.data;
    }

    public getToken() {
        return localStorage.getItem(this.tokenName);
    }

    public setToken(token: string) {
        localStorage.setItem(this.tokenName, token);
        this.setAuth(true);
        this.router.navigate(['']);
    }

    // This method will log the use out
    public logout() {
        localStorage.removeItem(this.tokenName);
        this.setAuth(false);
        this.router.navigate(['/login']);
    }

    public check() {
        let result = tokenNotExpired(this.tokenName, this.getToken());
        if (result !== this.lastCheck) {
            this.setAuth(result);
        }
        this.lastCheck = result;
        return result;
    }

}
