import {
    Component,
    OnInit
} from '@angular/core';
import { AuthService } from '../../core/auth/auth.service';
import { FormGroup , FormBuilder, Validators } from '@angular/forms';
import { Router } from '@angular/router';
import { UserService } from '../user/user.service';

@Component({

    selector: 'auth-page',

    styles: [`
    .login-register {
        width: 100%;
    }
    .app-title {
        font-size: 35px;
        margin-top: -15px;
    }
    .logo {
        cursor: default;
    }
    
    .login-sidebar {
        overflow: auto;
    }
    
    .login-sidebar .white-box {
        position: absolute;
        top: 10%;
        left: 0;
        right: 0;
        bottom: 25%;
    }
        
    .alert-danger{
        margin: 20px 0 0 0;
    }
`],

    templateUrl: './auth-page.component.html',

    host: {
      '[class.login]': 'true'
    }

})
export class AuthPageComponent implements OnInit {

    public loginForm: FormGroup;
    private isAuth = false;

    constructor(
        public auth: AuthService,
        public userService: UserService,
        public fb: FormBuilder,
        public router: Router
    ) {
    }

    public ngOnInit(): void {
        this.buildForm();
    }

    public buildForm(): void {
        this.loginForm = this.fb.group({
            email: ['',
                [
                    Validators.required,
                    Validators.pattern(
                        new RegExp(
                            /^(([^<>()\[\]\\.,;:\s@"]+(\.[^<>()\[\]\\.,;:\s@"]+)*)|(".+"))/.source +
                            /@((\[[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}])|(([a-zA-Z\-0-9]+\.)+[a-zA-Z]{2,}))$/.source
                        )
                    )
                ]
            ],
            password: ['', [
                    Validators.required,
                    Validators.minLength(6)
                ]
            ]
        });
    }

    public login() {
        if (!(this.loginForm.get('email').value && this.loginForm.get('password').value)) {
            return null;
        }

        return this.userService.loginUser(
            this.loginForm.get('email').value,
            this.loginForm.get('password').value
        ).subscribe((data) => {
            this.auth.setToken(data.json()['token']);

        }, () => {
            this.isAuth = true;
        });
    }
}
